# $Id: simulator.py 56 2005-09-03 20:46:55Z  $

import pygame

SIMULATOR_BACKGROUND = "../media/images/simulayout.bmp"
CAPACICATOR_EXPONENT_BASE = 2.71
BASE_CAPACITY = 0.172

TIME_POSITION = (151,267)
MONEY_POSITION = (155,343)
VOLTAGE_POSITION = (128,193)

CAPACICATOR_WIDTH = 24
CAPACICATOR_Y_POSITION = 39

CAPACICATOR_TOP = 26
CAPACICATOR_BOTTOM = 49
CAPACICATOR_LEFT = 43
CAPACICATOR_ON_SCREEN = 8

POWER_METER_HEIGHT = 53
POWER_METER_BASE = 105

RIGHT_ARROW_PLACEMENT = (256,25,14,25)
LEFT_ARROW_PLACEMENT = (16,25,18,24)
ADD_VOLTAGE_PLACEMENT = (222,183,19,25)
SUB_VOLTAGE_PLACEMENT = (251,183,19,25)
BATTERY_BUY_PRICE = 1500
BATTERY_SELL_PRICE = 1300

#enums
TARGET_MONEY = 'target_money'
TARGET_TIME = 'target_time'

def BlitAtMidpoint(dest,source,midpoint):
	sourceSize = source.get_size()
	targetPos = (midpoint[0]-sourceSize[0]/2, midpoint[1]-sourceSize[1]/2)
	dest.blit(source,targetPos)

class Simulator:	
	def __init__(self,host,options):
		self.host = host
		self.options = options
		
		self.Surface = pygame.Surface((300,400), 0, 24)

		self.RightRect = pygame.Rect(RIGHT_ARROW_PLACEMENT)
		self.LeftRect = pygame.Rect(LEFT_ARROW_PLACEMENT)
		self.AddVoltageRect = pygame.Rect(ADD_VOLTAGE_PLACEMENT)
		self.SubVoltageRect = pygame.Rect(SUB_VOLTAGE_PLACEMENT)

		self.font = pygame.font.SysFont("Tahoma",36,True,False)
		self.fontSmall = pygame.font.SysFont("Tahoma",14,True,False)
		self.backGround = pygame.image.load(SIMULATOR_BACKGROUND)

		self.money = 0
		self.time = 0
		self.batteries = 0
		self.items=[]
		
		self.offset = 0
		self.resistance = 1
		self.capacity = BASE_CAPACITY

	def getExponent(self):
		self.capacity = BASE_CAPACITY * (1 + self.batteries)
		self.resistance = 1.0 / (len(self.items)+1)
		return pow(CAPACICATOR_EXPONENT_BASE, (self.resistance * self.capacity))
	
	def getSurface(self):
		return self.Surface

	def sell_item(self,index):
		self.money += self.items[index].getCurrentPower()
		self.items = self.items[:index]+self.items[index+1:]

	#todo: convert to constants

	def handleEvent(self,event):
		"""returns true/false: whether the event was processed by the playfield"""
		if event.type == pygame.MOUSEBUTTONDOWN:
			if event.button == 1: #left button
				if event.pos[1] >= CAPACICATOR_TOP and event.pos[1] <= CAPACICATOR_BOTTOM:
					x = event.pos[0]-CAPACICATOR_LEFT
					if x>=0 and x<= 206:
						i = x/CAPACICATOR_WIDTH - self.offset
						if i>=0 and i<len(self.items):
							self.sell_item(i)
				if self.RightRect.collidepoint(*event.pos):
					self.offset -= 1
				if self.LeftRect.collidepoint(*event.pos):
					self.offset += 1
					if self.offset > 0:
						self.offset = 0
				if self.AddVoltageRect.collidepoint(*event.pos):
					if self.money >= BATTERY_BUY_PRICE:
						self.money -= BATTERY_BUY_PRICE
						self.batteries += 1	
				if self.SubVoltageRect.collidepoint(*event.pos):
					if self.batteries > 0:
						self.money += BATTERY_SELL_PRICE
						self.batteries -= 1
			
			elif event.button == 3: #right button
				pass
		return False
		
	def doFrame(self,timePassed):
		"""moves the player, according to the amount of time passed"""
		#the surface should be updated here

		self.Surface.blit(self.backGround,(0,0))
		
		self.time += timePassed
		textSurface = self.font.render(str(int(self.time)),True,(0,0,0))
		BlitAtMidpoint(self.Surface, textSurface, TIME_POSITION)
		
		textSurface = self.font.render(str(int(self.money))+"$",True,(0,0,0))
		BlitAtMidpoint(self.Surface, textSurface, MONEY_POSITION)
		
		textSurface = self.fontSmall.render(str(self.capacity)+"mvA",True,(0,0,0))
		BlitAtMidpoint(self.Surface, textSurface, VOLTAGE_POSITION)
		
		exponent = self.getExponent()
		for i in xrange(len(self.items)):
			item = self.items[i]
			item.setExponent(exponent)
			item.doFrame(timePassed)
			
			index = i + self.offset
			if index>=0 and index < CAPACICATOR_ON_SCREEN:
				position = (53+index*CAPACICATOR_WIDTH,CAPACICATOR_Y_POSITION)
				BlitAtMidpoint(self.Surface, item.getSurface(), position)
				
				power = item.getCurrent()
				power_meter=(49+index*CAPACICATOR_WIDTH, POWER_METER_BASE-power*POWER_METER_HEIGHT, 10, power*POWER_METER_HEIGHT)
				pygame.draw.rect(self.Surface, (0,255*power,0), power_meter)
		
		if self.money >= self.options[TARGET_MONEY]:
			self.host.WinLevel()
		if self.time >= self.options[TARGET_TIME]:
			self.host.LoseLevel()
			

	def itemCollectCallback(self,item):
		item.time = 0
		self.items.append(item)

#order of function calls:
#   init
#	   *handleEvent
#	   doFrame
#	   getSurface
